//------------------------------------------------------------------------------------
// runge_kutt.cpp
//     .   dy/dx=F(x,y)
//  -
// :  , nevsoft@mail.ru
// 24.05.2005
//------------------------------------------------------------------------------------
#include <math.h>
#include "runge_kutt.h"

namespace Mathematics
{

typedef std::pair<double, double> pair_of_doubles;

double RungeKuttStep(double x, double y, double h, double (*F)(double x, double y))
{
double K1 = h*F(x      ,y       );
double K2 = h*F(x+h/2.0,y+K1/2.0);
double K3 = h*F(x+h/2.0,y+K2/2.0);
double K4 = h*F(x+h    ,y+K3    );
return y+(K1+2.0*K2+2.0*K3+K4)/6.0;
}

bool RungeKutt(double x0, double y0, double xn, double h, double (*F)(double x, double y), std::vector<pair_of_doubles> &result) throw()
{
try
 {
 result.clear();
 int n = ceil((xn-x0)/h);
 result.reserve(n+1);
 double x = x0;
 double y = y0;
 result.push_back(std::pair<double,double>(x,y));
 for(int i=0;i<n;++i)
  {
  y = RungeKuttStep(x,y,h,F);
  x+=h;
  result.push_back(std::pair<double,double>(x,y));
  }
 }
catch(...)
 {
 return false;
 }
return true;
}

} // namespace Mathematics